"""
Awdible is a Python library for working as a Audible like.
"""

__VERSION__ = "0.2.5"
__DESCRIPTION__ = "Awdible - Just the best free version of audible"
__AUTHOR__ = "Alexandre Gazagnes"

# from .core import Awdible

# __all__ = ["Awdible"]
